package com.ruoyi.school.paper.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import cn.dev33.satoken.annotation.SaIgnore;
import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.school.paper.domain.vo.DbDictVo;
import com.ruoyi.school.paper.domain.bo.DbDictBo;
import com.ruoyi.school.paper.service.IDbDictService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 字典
 *
 * @author zyf
 * @date 2023-07-20
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/school-paper/dict")
@SaIgnore
public class DbDictController extends BaseController {

    private final IDbDictService iDbDictService;

    /**
     * 查询字典列表
     */
    @GetMapping("/list")
    public TableDataInfo<DbDictVo> list(DbDictBo bo, PageQuery pageQuery) {
        return iDbDictService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出字典列表
     */
    @Log(title = "字典", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(DbDictBo bo, HttpServletResponse response) {
        List<DbDictVo> list = iDbDictService.queryList(bo);
        ExcelUtil.exportExcel(list, "字典", DbDictVo.class, response);
    }

    /**
     * 获取字典详细信息
     *
     * @param id 主键
     */
    @GetMapping("/{id}")
    public R<DbDictVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iDbDictService.queryById(id));
    }

    /**
     * 新增字典
     */
    @Log(title = "字典", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody DbDictBo bo) {
        return toAjax(iDbDictService.insertByBo(bo));
    }

    /**
     * 修改字典
     */
    @Log(title = "字典", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody DbDictBo bo) {
        return toAjax(iDbDictService.updateByBo(bo));
    }

    /**
     * 删除字典
     *
     * @param ids 主键串
     */
    @Log(title = "字典", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iDbDictService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
