package com.ruoyi.school.paper.controller;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import cn.dev33.satoken.annotation.SaIgnore;
import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.school.paper.domain.vo.DbQuestionBankVo;
import com.ruoyi.school.paper.domain.bo.DbQuestionBankBo;
import com.ruoyi.school.paper.service.IDbQuestionBankService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 题库主
 *
 * @author zyf
 * @date 2023-07-20
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/school-paper/questionBank")
@SaIgnore
public class DbQuestionBankController extends BaseController {

    private final IDbQuestionBankService iDbQuestionBankService;

    /**
     * 查询题库主列表
     */
    @GetMapping("/list")
    public TableDataInfo<DbQuestionBankVo> list(DbQuestionBankBo bo, PageQuery pageQuery) {
        return iDbQuestionBankService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出题库主列表
     */
    @Log(title = "题库主", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(DbQuestionBankBo bo, HttpServletResponse response) {
        List<DbQuestionBankVo> list = iDbQuestionBankService.queryList(bo);
        ExcelUtil.exportExcel(list, "题库主", DbQuestionBankVo.class, response);
    }

    /**
     * 获取题库主详细信息
     *
     * @param id 主键
     */
    @GetMapping("/{id}")
    public R<DbQuestionBankVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(iDbQuestionBankService.queryById(id));
    }

    /**
     * 新增题库主
     */
    @Log(title = "题库主", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody DbQuestionBankBo bo) {
        return toAjax(iDbQuestionBankService.insertByBo(bo));
    }

    /**
     * 修改题库主
     */
    @Log(title = "题库主", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody DbQuestionBankBo bo) {
        return toAjax(iDbQuestionBankService.updateByBo(bo));
    }

    /**
     * 删除题库主
     *
     * @param ids 主键串
     */
    @Log(title = "题库主", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(iDbQuestionBankService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
}
