package com.ruoyi.school.paper.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.school.paper.domain.bo.DbDictBo;
import com.ruoyi.school.paper.domain.vo.DbDictVo;
import com.ruoyi.school.paper.domain.DbDict;
import com.ruoyi.school.paper.mapper.DbDictMapper;
import com.ruoyi.school.paper.service.IDbDictService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 字典Service业务层处理
 *
 * @author zyf
 * @date 2023-07-20
 */
@RequiredArgsConstructor
@Service
public class DbDictServiceImpl implements IDbDictService {

    private final DbDictMapper baseMapper;

    /**
     * 查询字典
     */
    @Override
    public DbDictVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询字典列表
     */
    @Override
    public TableDataInfo<DbDictVo> queryPageList(DbDictBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<DbDict> lqw = buildQueryWrapper(bo);
        Page<DbDictVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询字典列表
     */
    @Override
    public List<DbDictVo> queryList(DbDictBo bo) {
        LambdaQueryWrapper<DbDict> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<DbDict> buildQueryWrapper(DbDictBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<DbDict> lqw = Wrappers.lambdaQuery();
        lqw.eq(bo.getParentId() != null, DbDict::getParentId, bo.getParentId());
        lqw.eq(StringUtils.isNotBlank(bo.getType()), DbDict::getType, bo.getType());
        lqw.eq(StringUtils.isNotBlank(bo.getTypeValue()), DbDict::getTypeValue, bo.getTypeValue());
        lqw.eq(bo.getTypeStatus() != null, DbDict::getTypeStatus, bo.getTypeStatus());
        return lqw;
    }

    /**
     * 新增字典
     */
    @Override
    public Boolean insertByBo(DbDictBo bo) {
        DbDict add = BeanUtil.toBean(bo, DbDict.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改字典
     */
    @Override
    public Boolean updateByBo(DbDictBo bo) {
        DbDict update = BeanUtil.toBean(bo, DbDict.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(DbDict entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除字典
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
