package com.ruoyi.school.paper.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.school.paper.domain.bo.DbQuestionBankBo;
import com.ruoyi.school.paper.domain.vo.DbQuestionBankVo;
import com.ruoyi.school.paper.domain.DbQuestionBank;
import com.ruoyi.school.paper.mapper.DbQuestionBankMapper;
import com.ruoyi.school.paper.service.IDbQuestionBankService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 题库主Service业务层处理
 *
 * @author zyf
 * @date 2023-07-20
 */
@RequiredArgsConstructor
@Service
public class DbQuestionBankServiceImpl implements IDbQuestionBankService {

    private final DbQuestionBankMapper baseMapper;

    /**
     * 查询题库主
     */
    @Override
    public DbQuestionBankVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询题库主列表
     */
    @Override
    public TableDataInfo<DbQuestionBankVo> queryPageList(DbQuestionBankBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<DbQuestionBank> lqw = buildQueryWrapper(bo);
        Page<DbQuestionBankVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询题库主列表
     */
    @Override
    public List<DbQuestionBankVo> queryList(DbQuestionBankBo bo) {
        LambdaQueryWrapper<DbQuestionBank> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<DbQuestionBank> buildQueryWrapper(DbQuestionBankBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<DbQuestionBank> lqw = Wrappers.lambdaQuery();
        lqw.eq(bo.getCategoryId() != null, DbQuestionBank::getCategoryId, bo.getCategoryId());
        lqw.eq(bo.getSpecies() != null, DbQuestionBank::getSpecies, bo.getSpecies());
        lqw.eq(bo.getPhaseId() != null, DbQuestionBank::getPhaseId, bo.getPhaseId());
        lqw.eq(bo.getType() != null, DbQuestionBank::getType, bo.getType());
        lqw.eq(bo.getIntensity() != null, DbQuestionBank::getIntensity, bo.getIntensity());
        lqw.eq(StringUtils.isNotBlank(bo.getTopic()), DbQuestionBank::getTopic, bo.getTopic());
        lqw.eq(StringUtils.isNotBlank(bo.getRightAnswers()), DbQuestionBank::getRightAnswers, bo.getRightAnswers());
        lqw.eq(StringUtils.isNotBlank(bo.getRightAnswersPic()), DbQuestionBank::getRightAnswersPic, bo.getRightAnswersPic());
        lqw.eq(StringUtils.isNotBlank(bo.getResolveCourse()), DbQuestionBank::getResolveCourse, bo.getResolveCourse());
        lqw.eq(StringUtils.isNotBlank(bo.getResolveVideo()), DbQuestionBank::getResolveVideo, bo.getResolveVideo());
        lqw.eq(bo.getStatus() != null, DbQuestionBank::getStatus, bo.getStatus());
        return lqw;
    }

    /**
     * 新增题库主
     */
    @Override
    public Boolean insertByBo(DbQuestionBankBo bo) {
        DbQuestionBank add = BeanUtil.toBean(bo, DbQuestionBank.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改题库主
     */
    @Override
    public Boolean updateByBo(DbQuestionBankBo bo) {
        DbQuestionBank update = BeanUtil.toBean(bo, DbQuestionBank.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(DbQuestionBank entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除题库主
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
