package com.ruoyi.school.paper.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.domain.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import com.ruoyi.school.paper.domain.bo.DbTestPaperBo;
import com.ruoyi.school.paper.domain.vo.DbTestPaperVo;
import com.ruoyi.school.paper.domain.DbTestPaper;
import com.ruoyi.school.paper.mapper.DbTestPaperMapper;
import com.ruoyi.school.paper.service.IDbTestPaperService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 试卷Service业务层处理
 *
 * @author zyf
 * @date 2023-07-20
 */
@RequiredArgsConstructor
@Service
public class DbTestPaperServiceImpl implements IDbTestPaperService {

    private final DbTestPaperMapper baseMapper;

    /**
     * 查询试卷
     */
    @Override
    public DbTestPaperVo queryById(Long id){
        return baseMapper.selectVoById(id);
    }

    /**
     * 查询试卷列表
     */
    @Override
    public TableDataInfo<DbTestPaperVo> queryPageList(DbTestPaperBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<DbTestPaper> lqw = buildQueryWrapper(bo);
        Page<DbTestPaperVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询试卷列表
     */
    @Override
    public List<DbTestPaperVo> queryList(DbTestPaperBo bo) {
        LambdaQueryWrapper<DbTestPaper> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<DbTestPaper> buildQueryWrapper(DbTestPaperBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<DbTestPaper> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getTestName()), DbTestPaper::getTestName, bo.getTestName());
        lqw.eq(bo.getCategoryId() != null, DbTestPaper::getCategoryId, bo.getCategoryId());
        lqw.eq(bo.getPhaseId() != null, DbTestPaper::getPhaseId, bo.getPhaseId());
        lqw.eq(bo.getClassId() != null, DbTestPaper::getClassId, bo.getClassId());
        lqw.eq(bo.getTotalPoints() != null, DbTestPaper::getTotalPoints, bo.getTotalPoints());
        lqw.eq(bo.getStatus() != null, DbTestPaper::getStatus, bo.getStatus());
        return lqw;
    }

    /**
     * 新增试卷
     */
    @Override
    public Boolean insertByBo(DbTestPaperBo bo) {
        DbTestPaper add = BeanUtil.toBean(bo, DbTestPaper.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setId(add.getId());
        }
        return flag;
    }

    /**
     * 修改试卷
     */
    @Override
    public Boolean updateByBo(DbTestPaperBo bo) {
        DbTestPaper update = BeanUtil.toBean(bo, DbTestPaper.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(DbTestPaper entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 批量删除试卷
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteBatchIds(ids) > 0;
    }
}
